rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("twang")
library("mgcv")
library("survey")
library("splines")
source("ResBal.R")

tmp_fun_a <- function(x){ifelse(x %in% -seq(1, 5), NA, x)}
tmp_fun_b <- compose(as.numeric, as.character)
percentile_rank <- function(x){rank(x, na.last = "keep")/sum(!is.na(x))}

Mode <- function(x) {
  x <- x[complete.cases(x)]
  ux <- unique(x)
  ux[which.max(tabulate(match(x, ux)))]
}

wtd_rank <- function (x, weights = NULL, normwt = FALSE, na.rm = TRUE){
  if (!length(weights))
    return(rank(x, na.last = if (na.rm) NA else TRUE))
  tab <- Hmisc::wtd.table(x, weights, normwt = normwt, na.rm = na.rm)
  freqs <- tab$sum.of.weights
  r <- cumsum(freqs) - 0.5 * (freqs - 1)
  stats::approx(tab$x, r, xout = x, rule = 2)$y
}

load("nlsy79_samples.RData")

origin_bs <- ns(nlsy79_ed_b$origin, df = 3)

res <- function(y, x, w) {
  if(identical(sort(unique(y)), c(0, 1))){
    return(y - glm.fit(y = y, x = x, weights = w, family = binomial("logit"))[["fitted.values"]])
  } else return(lm.wfit(y = y, x = x, w = w)[["residuals"]])
}

nlsy79_ed_b <- cbind(nlsy79_ed_b, origin_bs) %>%
  `names<-`(c(names(.)[1:(ncol(.)-3)], paste("origin_bs", 1:3, sep = ""))) %>%
  mutate(female = as.numeric(sex == 2)) %>%
  mutate_each(funs(res = res(., x = cbind(1, origin) , w = weights)), c(test_score:num_sibs, female)) %>%
  mutate_each(funs(origin = `*`(., origin)), test_score_res:female_res)  %>%
  mutate_each(funs(college = `*`(., college)), test_score_res:female_res) %>%
  mutate_each(funs(origin = `*`(., origin)), c(test_score_res_college:female_res_college))

summary(nlsy79_ed_b)

# ipw

logit_sim <- glm(college ~ origin_bs1 + origin_bs2 + origin_bs3, family = binomial("logit"),
                 data = nlsy79_ed_b, weights = weights)

logit_full <- glm(college ~ origin_bs1 + origin_bs2 + origin_bs3 + test_score + educ_exp +
                    father + black + hispanic + female + urban + educ_mom + num_sibs,
                    family = binomial("logit"), data = nlsy79_ed_b, weights = weights)

logit_probs_sim <- cbind(1-fitted(logit_sim), fitted(logit_sim))
logit_probs_full <- cbind(1-plogis(predict(logit_full, nlsy79_ed_b)),
                          plogis(predict(logit_full, nlsy79_ed_b)))

# data 

college_grad <- outer(nlsy79_ed_b$college, c(FALSE, TRUE), `==`)

nlsy79_ed3_b <- nlsy79_ed_b %>% 
  mutate(wt_adjust_ipw = rowSums(college_grad * logit_probs_sim)/rowSums(college_grad * logit_probs_full),
         wt_adjusted_ipw = weights * wt_adjust_ipw) %>%
  group_by(college) %>%
  mutate(wt_adjusted_ipw = wt_adjusted_ipw/sum(wt_adjusted_ipw)*sum(weights)) %>%
  ungroup()

# balance in E(X|O) estimated as the slope of lm(X ~ O)

lm_coef <- function(x, y, w) lm.wfit(x, y, w)[["coefficients"]][2]

conditional <- nlsy79_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                 c(test_score:num_sibs, female)) %>% `[`(, -1) %>% t()

overall <- nlsy79_ed3_b %>% summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                                           c(test_score:num_sibs, female)) %>% t()

reweighted <- nlsy79_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = wt_adjusted_ipw)),
                 c(test_score:num_sibs, female)) %>% `[`(, -1) %>% t()

balance_b <- cbind(overall, conditional, reweighted)

# prep

data_new <- expand.grid(college = c(FALSE, TRUE), origin = seq(0, 1, 0.01))
origin_bs_simp <- ns(data_new$origin, knots = attr(origin_bs, "knots"))
colnames(origin_bs_simp) <- paste("origin_bs", 1:ncol(origin_bs_simp), sep = "")
data_new <- cbind(data_new, origin_bs_simp)

formula1 <- dest ~ origin + college + origin*college
formula2 <- dest ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
  origin_bs1*college + origin_bs2*college + origin_bs3*college

# check balance plots

var2bal <- c("female", "test_score", "educ_exp", "father", "hispanic",
             "black", "urban", "educ_mom", "num_sibs")

formulas <- lapply(var2bal, compose(as.formula, paste), " ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
                   origin_bs1*college + origin_bs2*college + origin_bs3*college")

nlsy79_ed3_b_augmented <- nlsy79_ed3_b

nlsy79_ed3_b_augmented <- nlsy79_ed3_b %>% filter(college==TRUE) %>%
  mutate(college = FALSE) %>% rbind(nlsy79_ed3_b)

design_naive <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_ed3_b_augmented)
design_ipw <- svydesign(ids = ~1, weights = ~ wt_adjusted_ipw, data = nlsy79_ed3_b_augmented)

svyglm_naive <- partial(svyglm, design = design_naive)
svyglm_naive_logit <- partial(svyglm, family = binomial("logit"), design = design_naive)
svyglms_naive <- list(svyglm_naive_logit, svyglm_naive, svyglm_naive, svyglm_naive_logit, svyglm_naive_logit,
                      svyglm_naive_logit, svyglm_naive_logit, svyglm_naive, svyglm_naive)

svyglm_ipw <- partial(svyglm, design = design_ipw)
svyglm_ipw_logit <- partial(svyglm, family = binomial("logit"), design = design_ipw)
svyglms_ipw <- list(svyglm_ipw_logit, svyglm_ipw, svyglm_ipw, svyglm_ipw_logit, svyglm_ipw_logit,
                     svyglm_ipw_logit, svyglm_ipw_logit, svyglm_ipw, svyglm_ipw)

cov_b_before_ipw <- Map(function(f, x) f(x), svyglms_naive, formulas)
cov_b_after_ipw <- Map(function(f, x) f(x), svyglms_ipw, formulas)

plot_b_before_ipw <- lapply(cov_b_before_ipw, function(x) cbind(data_new, predict(x, data_new, type = "response")))
plot_b_after_ipw <- lapply(cov_b_after_ipw, function(x) cbind(data_new, predict(x, data_new, type = "response")))

# msm naive

design_naive <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_ed3_b)
msm_b_naive <- svyglm(formula1, design = design_naive)
msm_b_naive_plot <- cbind(data_new, predict(msm_b_naive, newdata = data_new))

summary(msm_b_naive)

# with ipw weights

design_ipw <- svydesign(ids = ~1, weights = ~ wt_adjusted_ipw, data = nlsy79_ed3_b)
msm_b_ipw <- svyglm(formula1, design = design_ipw)
msm_b_ipw_plot <- cbind(data_new, predict(msm_b_ipw, newdata = data_new))

summary(msm_b_ipw)

save.image("results_full_ipw.RData")

