rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("twang")
library("mgcv")
library("survey")
library("splines")
library("Matching")

tmp_fun_a <- function(x){ifelse(x %in% -seq(1, 5), NA, x)}
tmp_fun_b <- compose(as.numeric, as.character)
percentile_rank <- function(x){rank(x, na.last = "keep")/sum(!is.na(x))}

Mode <- function(x) {
  x <- x[complete.cases(x)]
  ux <- unique(x)
  ux[which.max(tabulate(match(x, ux)))]
}

wtd_rank <- function (x, weights = NULL, normwt = FALSE, na.rm = TRUE){
  if (!length(weights))
    return(rank(x, na.last = if (na.rm) NA else TRUE))
  tab <- Hmisc::wtd.table(x, weights, normwt = normwt, na.rm = na.rm)
  freqs <- tab$sum.of.weights
  r <- cumsum(freqs) - 0.5 * (freqs - 1)
  stats::approx(tab$x, r, xout = x, rule = 2)$y
}

load("nlsy79_samples.RData")

origin_bs <- ns(nlsy79_ed_b$origin, df = 3)

nlsy79_ed3_b <- cbind(nlsy79_ed_b, origin_bs) %>%
  `names<-`(c(names(.)[1:(ncol(.)-3)], paste("origin_bs", 1:3, sep = ""))) %>%
  mutate(female = as.numeric(sex == 2),
         index = 1:nrow(.),
         nocollege = 1 - college) %>%
  rename(nlsy_weight = weights)

logit_full <- glm(college ~ origin_bs1 + origin_bs2 + origin_bs3 + test_score + educ_exp +
                    father + black + hispanic + female + urban + educ_mom + num_sibs,
                  family = binomial("logit"), data = nlsy79_ed3_b, weights = nlsy_weight)

logit <- function(x) log(x/(1-x))
nlsy79_ed3_b$ps <- fitted(logit_full)
nlsy79_ed3_b$lps <- logit(nlsy79_ed3_b$ps)

cov_names <- c("origin", "female", "test_score", "educ_exp", "father",
               "hispanic", "black", "urban", "educ_mom", "num_sibs")

formula_college <- as.formula(paste("college ~ ", paste(cov_names, collapse= "+")))
formula_nocollege <- as.formula(paste("nocollege ~ ", paste(cov_names, collapse= "+")))

# matched sample for treated units

match_college <- with(nlsy79_ed3_b, Match(Tr = college, Weight = 1, M = 1, X = nlsy79_ed3_b["ps"]))

summary(match_college)

pair_college <- with(match_college, data.frame(index.treated, index.control, weights)) %>%
  left_join(nlsy79_ed3_b[, c("index", "nlsy_weight")], by = c("index.treated" = "index")) %>%
  rename(nlsy_weight2 = nlsy_weight)

treated_college <- pair_college %>%
  right_join(nlsy79_ed3_b, ., by = c("index" = "index.treated", "nlsy_weight" = "nlsy_weight2")) %>%
  dplyr::select(-index.control)

control_college <- pair_college %>%
  right_join(nlsy79_ed3_b, ., by = c("index" = "index.control")) %>%
  dplyr::select(-index.treated, -nlsy_weight2)

matched_college <- rbind(treated_college, control_college) %>%
  # mutate(weights = weights * nlsy_weight) %>%
  group_by(id) %>% mutate(weights = sum(weights)) %>%
  `[`(!duplicated(.),) %>% ungroup()

matched_college %>% group_by(college) %>%
  summarise(total = sum(weights))

# matched sample for untreated units

match_nocollege <- with(nlsy79_ed3_b, Match(Tr = nocollege, Weight = 1, M = 1, X = nlsy79_ed3_b["ps"]))

summary(match_nocollege)

pair_nocollege <- with(match_nocollege, data.frame(index.treated, index.control, weights)) %>%
  left_join(nlsy79_ed3_b[, c("index", "nlsy_weight")], by = c("index.treated" = "index")) %>%
  rename(nlsy_weight2 = nlsy_weight)

treated_nocollege <- pair_nocollege %>%
  right_join(nlsy79_ed3_b, ., by = c("index" = "index.treated", "nlsy_weight" = "nlsy_weight2")) %>%
  dplyr::select(-index.control)

control_nocollege <- pair_nocollege %>%
  right_join(nlsy79_ed3_b, ., by = c("index" = "index.control")) %>%
  dplyr::select(-index.treated, -nlsy_weight2)

matched_nocollege <- rbind(treated_nocollege, control_nocollege) %>%
  # mutate(weights = weights * nlsy_weight) %>%
  group_by(id) %>% mutate(weights = sum(weights)) %>%
  `[`(!duplicated(.),) %>% ungroup()

matched_nocollege %>% group_by(nocollege) %>%
  summarise(total = sum(weights))

identical(names(matched_college), names(matched_nocollege))

nlsy79_matched <- rbind(matched_nocollege, matched_college) %>%
  group_by(id) %>% mutate(weights = sum(weights)) %>%
  `[`(!duplicated(.),) %>% ungroup() %>%
  mutate(weights = weights * nlsy_weight)

# balance plots

data_new <- expand.grid(college = c(FALSE, TRUE), origin = seq(0, 1, 0.01))
origin_bs_simp <- ns(data_new$origin, knots = attr(origin_bs, "knots"))
colnames(origin_bs_simp) <- paste("origin_bs", 1:ncol(origin_bs_simp), sep = "")
data_new <- cbind(data_new, origin_bs_simp)

formula1 <- dest ~ origin + college + origin*college
formula2 <- dest ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
  origin_bs1*college + origin_bs2*college + origin_bs3*college

var2bal <- c("female", "test_score", "educ_exp", "father", "hispanic",
             "black", "urban", "educ_mom", "num_sibs")

formulas <- lapply(var2bal, compose(as.formula, paste), " ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
                   origin_bs1*college + origin_bs2*college + origin_bs3*college")

nlsy79_ed3_b_augmented <- nlsy79_ed3_b %>% filter(college==TRUE) %>%
  mutate(college = FALSE) %>% rbind(nlsy79_ed3_b)

nlsy79_matched_augmented <- nlsy79_matched %>% filter(college==TRUE) %>%
  mutate(college = FALSE) %>% rbind(nlsy79_matched)

design_naive <- svydesign(ids = ~1, weights = ~ nlsy_weight, data = nlsy79_ed3_b_augmented)
design_psm <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_matched_augmented)

svyglm_naive <- partial(svyglm, design = design_naive)
svyglm_naive_logit <- partial(svyglm, family = binomial("logit"), design = design_naive)
svyglms_naive <- list(svyglm_naive_logit, svyglm_naive, svyglm_naive, svyglm_naive_logit, svyglm_naive_logit,
                      svyglm_naive_logit, svyglm_naive_logit, svyglm_naive, svyglm_naive)

svyglm_psm <- partial(svyglm, design = design_psm)
svyglm_psm_logit <- partial(svyglm, family = binomial("logit"), design = design_psm)
svyglms_psm <- list(svyglm_psm_logit, svyglm_psm, svyglm_psm, svyglm_psm_logit, svyglm_psm_logit,
                    svyglm_psm_logit, svyglm_psm_logit, svyglm_psm, svyglm_psm)

cov_b_before_psm <- Map(function(f, x) f(x), svyglms_naive, formulas)
cov_b_after_psm <- Map(function(f, x) f(x), svyglms_psm, formulas)

plot_b_before_psm <- lapply(cov_b_before_psm, function(x) cbind(data_new, predict(x, data_new, type = "response")))
plot_b_after_psm <- lapply(cov_b_after_psm, function(x) cbind(data_new, predict(x, data_new, type = "response")))

# msm naive

design_naive <- svydesign(ids = ~1, weights = ~ nlsy_weight, data = nlsy79_ed3_b)
msm_b_naive <- svyglm(formula1, design = design_naive)
msm_b_naive_plot <- cbind(data_new, predict(msm_b_naive, newdata = data_new))

summary(msm_b_naive)

# with psm weights

design_psm <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_matched)
msm_b_psm <- svyglm(formula1, design = design_psm)
msm_b_psm_plot <- cbind(data_new, predict(msm_b_psm, newdata = data_new))

summary(msm_b_psm)

save.image("results_full_psm.RData")

