rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("mgcv")
library("survey")
library("splines")
load("results_full.RData")

# demean function

demean <- function(x, w) x - weighted.mean(x, w)

# baseline data and simulated data 

nlsy79_base <- nlsy79_ed_b %>% filter(tfinc_dest_b >= 1000) %>%
  dplyr::select(origin, female, black, hispanic, educ_mom, father, num_sibs, urban, test_score, educ_exp,
                college, tfinc_dest_b, dest, weights) %>%
  mutate(college = as.numeric(college)) %>%
  mutate_at(., vars(origin:educ_exp), funs(demean), w = .[["weights"]])

nlsy79_sim <- nlsy79_base %>% dplyr::select(-college, -dest) %>%
  replicate(100, ., simplify = FALSE) %>% Reduce(rbind, .)

# college completion and income model

college_mod <- gam(college ~ s(origin, test_score) + female + black + hispanic +
                     s(educ_mom) + father + s(num_sibs) + urban + s(educ_exp),
                   weights = weights, family = binomial("logit"), data = nlsy79_base)

income_mod <- gam(log(tfinc_dest_b) ~ s(college, origin) + s(college, test_score) + s(origin, test_score) +
                    female + black + hispanic + s(educ_mom) + father + s(num_sibs) + urban + s(educ_exp),
                  weights = weights, data = nlsy79_base)

summary(income_mod)

# use college_mod to simulate college completion for nlsy_sim

nlsy79_sim$college <- rbinom(n = nrow(nlsy79_sim), size = 1,
                             prob = predict(college_mod, newdata = nlsy79_sim, type = "response"))
                            
nlsy79_sim$logincome <- rnorm(n = nrow(nlsy79_sim),
                              mean = predict(income_mod, newdata = nlsy79_sim),
                              sd = sigma(income_mod))

nlsy79_sim$dest <- wtd_rank(nlsy79_sim$logincome, weights = nlsy79_sim$weights, normwt = TRUE)/nrow(nlsy79_sim)

# rank-rank regression

rr_actual <- lm(dest ~ origin, weights = weights, data = nlsy79_ed_b)

rr_sim <- lm(dest ~ origin, weights = weights, data = nlsy79_sim)

summary(income_mod)

summary(rr_sim)

# simulation 

alphas <- seq(-6, 6, 0.2)

out <- matrix(NA, nrow = length(alphas), ncol = 4)

i <- 0

for (alpha in alphas){
  
  i <- i + 1
  
  college_mod$coefficients[1] <- alpha
  
  nlsy79_sim$college <- sapply(predict(college_mod, newdata = nlsy79_sim, type = "response"), rbinom, n = 1, size = 1)
  
  nlsy79_sim$logincome <- rnorm(nrow(nlsy79_sim), mean = predict(income_mod, newdata = nlsy79_sim), sd = sigma(income_mod))
  
  nlsy79_sim$dest <- wtd_rank(nlsy79_sim$logincome, weights = nlsy79_sim$weights, normwt = TRUE)/nrow(nlsy79_sim)
  
  rr_sim <- lm(dest ~ origin, weights = weights, data = nlsy79_sim)
  
  rr0_sim <- lm(dest ~ origin, weights = weights, data = nlsy79_sim, subset = (college == 0))
  
  rr1_sim <- lm(dest ~ origin, weights = weights, data = nlsy79_sim, subset = (college == 1))
  
  out[i, 1] <- weighted.mean(nlsy79_sim$college, nlsy79_sim$weights)
  
  out[i, 2] <- rr_sim$coef[2]
  
  out[i, 3] <- rr0_sim$coef[2]
    
  out[i, 4] <- rr1_sim$coef[2]
  
  print(round(out[i, ], 3))
}

save.image("college_expansion.RData")
